@@ -374,7 +374,7 @@ br_status bv_rewriter::rw_leq_concats(bool is_signed, expr * _a, expr * _b, expr
                 ptr_buffer<expr> new_args;
                 new_args.push_back(mk_numeral(af, af_sz - sz_min));
                 for (unsigned i = 1; i < numa; ++i) new_args.push_back(a->get_arg(i));
-                new_a = concat(new_args.size(), new_args.c_ptr());
+                new_a = concat(new_args.size(), new_args.data());
             } else {
                 new_a = concat(numa - 1, a->get_args() + 1);
             }
@@ -382,7 +382,7 @@ br_status bv_rewriter::rw_leq_concats(bool is_signed, expr * _a, expr * _b, expr
                 ptr_buffer<expr> new_args;
                 new_args.push_back(mk_numeral(bf, bf_sz - sz_min));
                 for (unsigned i = 1; i < numb; ++i) new_args.push_back(b->get_arg(i));
-                new_b = concat(new_args.size(), new_args.c_ptr());
+                new_b = concat(new_args.size(), new_args.data());
             } else {
                 new_b = concat(numb - 1, b->get_args() + 1);
             }
@@ -659,7 +659,7 @@ unsigned bv_rewriter::propagate_extract(unsigned high, expr * arg, expr_ref & re
                 new_concat_args.push_back(new_first);
                 for (unsigned j = 1; j < conc_num; ++j)
                     new_concat_args.push_back(to_app(curr)->get_arg(j));
-                new_arg = m_util.mk_concat(new_concat_args.size(), new_concat_args.c_ptr());
+                new_arg = m_util.mk_concat(new_concat_args.size(), new_concat_args.data());
             } else {
                 // remove first element of concat
                 expr * const * const old_conc_args = to_app(curr)->get_args();
@@ -675,7 +675,7 @@ unsigned bv_rewriter::propagate_extract(unsigned high, expr * arg, expr_ref & re
         }
         if (new_arg) new_args.push_back(new_arg);
     }
-    result = m().mk_app(get_fid(), a->get_decl()->get_decl_kind(), new_args.size(), new_args.c_ptr());
+    result = m().mk_app(get_fid(), a->get_decl()->get_decl_kind(), new_args.size(), new_args.data());
     SASSERT(m_util.is_bv(result));
     return removable;
 }
@@ -755,11 +755,11 @@ br_status bv_rewriter::mk_extract(unsigned high, unsigned low, expr * arg, expr_
                     }
                     if (idx == low) {
                         new_args.push_back(curr);
-                        result = m_util.mk_concat(new_args.size(), new_args.c_ptr());
+                        result = m_util.mk_concat(new_args.size(), new_args.data());
                         return used_extract ? BR_REWRITE2 : BR_DONE;
                     }
                     new_args.push_back(m_mk_extract(curr_sz - 1, low - idx, curr));
-                    result = m_util.mk_concat(new_args.size(), new_args.c_ptr());
+                    result = m_util.mk_concat(new_args.size(), new_args.data());
                     return BR_REWRITE2;
                 }
                 UNREACHABLE();
@@ -779,7 +779,7 @@ br_status bv_rewriter::mk_extract(unsigned high, unsigned low, expr * arg, expr_
             expr * curr = to_app(arg)->get_arg(i);
             new_args.push_back(m_mk_extract(high, low, curr));
         }
-        result = m().mk_app(get_fid(), to_app(arg)->get_decl()->get_decl_kind(), new_args.size(), new_args.c_ptr());
+        result = m().mk_app(get_fid(), to_app(arg)->get_decl()->get_decl_kind(), new_args.size(), new_args.data());
         return BR_REWRITE2;
     }
 
@@ -855,6 +855,16 @@ br_status bv_rewriter::mk_bv_shl(expr * arg1, expr * arg2, expr_ref & result) {
         return BR_REWRITE2;
     }
 
+    expr* x = nullptr, *y = nullptr;    
+    if (m_util.is_bv_shl(arg1, x, y)) {
+        expr_ref sum(m_util.mk_bv_add(y, arg2), m());
+        expr_ref cond(m_util.mk_ule(y, sum), m());
+        result = m().mk_ite(cond, 
+                            m_util.mk_bv_shl(x, sum),
+                            mk_numeral(0, bv_size));
+        return BR_REWRITE3;
+    }
+
     return BR_FAILED;
 }
 
@@ -1415,19 +1425,19 @@ br_status bv_rewriter::mk_bv2int(expr * arg, expr_ref & result) {
             args[i] = std::move(tmp);
             sz += get_bv_size(to_app(arg)->get_arg(i));
         }
-        result = m_autil.mk_add(args.size(), args.c_ptr());
+        result = m_autil.mk_add(args.size(), args.data());
         return BR_REWRITE2;
     }
     if (is_mul_no_overflow(arg)) {
         expr_ref_vector args(m());
         for (expr* x : *to_app(arg)) args.push_back(m_util.mk_bv2int(x));
-        result = m_autil.mk_mul(args.size(), args.c_ptr());
+        result = m_autil.mk_mul(args.size(), args.data());
         return BR_REWRITE2;
     }
     if (is_add_no_overflow(arg)) {
         expr_ref_vector args(m());
         for (expr* x : *to_app(arg)) args.push_back(m_util.mk_bv2int(x));
-        result = m_autil.mk_add(args.size(), args.c_ptr());
+        result = m_autil.mk_add(args.size(), args.data());
         return BR_REWRITE2;
     }
 
@@ -1436,18 +1446,34 @@ br_status bv_rewriter::mk_bv2int(expr * arg, expr_ref & result) {
 
 
 bool bv_rewriter::is_mul_no_overflow(expr* e) {
-    if (!m_util.is_bv_mul(e)) return false;
+    if (!m_util.is_bv_mul(e)) 
+        return false;
     unsigned sz = get_bv_size(e);
     unsigned sum = 0;
-    for (expr* x : *to_app(e)) sum += sz-num_leading_zero_bits(x);
-    return sum < sz;
+    for (expr* x : *to_app(e)) 
+        sum += sz - num_leading_zero_bits(x);
+    if (sum > sz + 1)
+        return false;
+    if (sum <= sz)
+        return true;
+    rational v;
+    unsigned shift;
+    for (expr* x : *to_app(e)) 
+        if (m_util.is_numeral(x, v) && v.is_power_of_two(shift))
+            return true;
+    return false;
 }
 
 bool bv_rewriter::is_add_no_overflow(expr* e) {
-    if (!is_add(e)) return false;
-    for (expr* x : *to_app(e)) {
-        if (0 == num_leading_zero_bits(x)) return false;
-    }
+    if (!is_add(e)) 
+        return false;
+    unsigned num_args = to_app(e)->get_num_args();
+    if (num_args <= 1)
+        return true;
+    num_args -= 2;
+    for (expr* x : *to_app(e)) 
+        if (num_args >= num_leading_zero_bits(x)) 
+            return false;
     return true;
 }
 
@@ -1528,7 +1554,7 @@ br_status bv_rewriter::mk_concat(unsigned num_args, expr * const * args, expr_re
         result = new_args.back();
         return fused_extract ? BR_REWRITE1 : BR_DONE;
     }
-    result = m_util.mk_concat(new_args.size(), new_args.c_ptr());
+    result = m_util.mk_concat(new_args.size(), new_args.data());
     if (fused_extract)
         return BR_REWRITE2;
     else if (expanded)
@@ -1574,7 +1600,7 @@ br_status bv_rewriter::mk_sign_extend(unsigned n, expr * arg, expr_ref & result)
         for (unsigned i = 0; i < n; i++)
             args.push_back(sign);
         args.push_back(arg);
-        result = m_util.mk_concat(args.size(), args.c_ptr());
+        result = m_util.mk_concat(args.size(), args.data());
         return BR_REWRITE2;
     }
 
@@ -1589,7 +1615,7 @@ br_status bv_rewriter::mk_repeat(unsigned n, expr * arg, expr_ref & result) {
     ptr_buffer<expr> args;
     for (unsigned i = 0; i < n; i++)
         args.push_back(arg);
-    result = m_util.mk_concat(args.size(), args.c_ptr());
+    result = m_util.mk_concat(args.size(), args.data());
     return BR_REWRITE1;
 }
 
@@ -1617,7 +1643,7 @@ br_status bv_rewriter::mk_bv_or(unsigned num, expr * const * args, expr_ref & re
         if (flat_args.size() != num) {
             flattened = true;
             num  = flat_args.size();
-            args = flat_args.c_ptr();
+            args = flat_args.data();
         }
     }
 
@@ -1696,7 +1722,7 @@ br_status bv_rewriter::mk_bv_or(unsigned num, expr * const * args, expr_ref & re
                 if (j != high)
                     non_zero_args.push_back(m_mk_extract(high, j+1, concat1));
             }
-            result = m_util.mk_concat(non_zero_args.size(), non_zero_args.c_ptr());
+            result = m_util.mk_concat(non_zero_args.size(), non_zero_args.data());
             return BR_REWRITE2;
         }
     }
@@ -1732,7 +1758,7 @@ br_status bv_rewriter::mk_bv_or(unsigned num, expr * const * args, expr_ref & re
             }
         }
         std::reverse(exs.begin(), exs.end());
-        result = m_util.mk_concat(exs.size(), exs.c_ptr());
+        result = m_util.mk_concat(exs.size(), exs.data());
         TRACE("mask_bug",
               tout << "(assert (distinct (bvor (_ bv" << old_v1 << " " << sz << ")\n" << mk_ismt2_pp(t, m()) << ")\n";
               tout << mk_ismt2_pp(result, m()) << "))\n";);
@@ -1757,10 +1783,10 @@ br_status bv_rewriter::mk_bv_or(unsigned num, expr * const * args, expr_ref & re
     default:
         if (m_bv_sort_ac)
             std::sort(new_args.begin(), new_args.end(), ast_to_lt());
-        if (distribute_concat(OP_BOR, new_args.size(), new_args.c_ptr(), result)) {
+        if (distribute_concat(OP_BOR, new_args.size(), new_args.data(), result)) {
             return BR_REWRITE3;
         }
-        result = m_util.mk_bv_or(new_args.size(), new_args.c_ptr());
+        result = m_util.mk_bv_or(new_args.size(), new_args.data());
         return BR_DONE;
     }
 }
@@ -1789,7 +1815,7 @@ br_status bv_rewriter::mk_bv_xor(unsigned num, expr * const * args, expr_ref & r
         if (flat_args.size() != num) {
             flattened = true;
             num  = flat_args.size();
-            args = flat_args.c_ptr();
+            args = flat_args.data();
         }
     }
 
@@ -1877,11 +1903,11 @@ br_status bv_rewriter::mk_bv_xor(unsigned num, expr * const * args, expr_ref & r
                 low = i;
             }
         }
-        std::reverse(exs.c_ptr(), exs.c_ptr() + exs.size());
+        std::reverse(exs.data(), exs.data() + exs.size());
         if (exs.size() == 1)
             result = exs[0];
         else
-            result = m_util.mk_concat(exs.size(), exs.c_ptr());
+            result = m_util.mk_concat(exs.size(), exs.data());
         return BR_REWRITE3;
     }
 
@@ -1933,10 +1959,10 @@ br_status bv_rewriter::mk_bv_xor(unsigned num, expr * const * args, expr_ref & r
     default:
         if (m_bv_sort_ac)
             std::sort(new_args.begin(), new_args.end(), ast_to_lt());
-        if (distribute_concat(OP_BXOR, new_args.size(), new_args.c_ptr(), result)) {
+        if (distribute_concat(OP_BXOR, new_args.size(), new_args.data(), result)) {
             return BR_REWRITE3;
         }
-        result = m_util.mk_bv_xor(new_args.size(), new_args.c_ptr());
+        result = m_util.mk_bv_xor(new_args.size(), new_args.data());
         return BR_DONE;
     }
 }
@@ -1953,8 +1979,8 @@ bool bv_rewriter::distribute_concat(decl_kind k, unsigned n, expr* const* args,
                 args1.push_back(m_mk_extract(sz2 - 1, sz2 - sz1, args[j]));
                 args2.push_back(m_mk_extract(sz2 - sz1 - 1, 0, args[j]));
             }
-            expr* arg1 = m().mk_app(get_fid(), k, args1.size(), args1.c_ptr());
-            expr* arg2 = m().mk_app(get_fid(), k, args2.size(), args2.c_ptr());
+            expr* arg1 = m().mk_app(get_fid(), k, args1.size(), args1.data());
+            expr* arg2 = m().mk_app(get_fid(), k, args2.size(), args2.data());
             result = m_util.mk_concat(arg1, arg2);
             return true;
         }
@@ -1981,7 +2007,7 @@ br_status bv_rewriter::mk_bv_not(expr * arg, expr_ref & result) {
         for (expr* a : *to_app(arg)) {
             new_args.push_back(m_util.mk_bv_not(a));
         }
-        result = m_util.mk_concat(new_args.size(), new_args.c_ptr());
+        result = m_util.mk_concat(new_args.size(), new_args.data());
         return BR_REWRITE2;
     }
 
@@ -2017,7 +2043,7 @@ br_status bv_rewriter::mk_bv_and(unsigned num, expr * const * args, expr_ref & r
         new_args.push_back(m_util.mk_bv_not(args[i]));
     }
     SASSERT(num == new_args.size());
-    result = m_util.mk_bv_not(m_util.mk_bv_or(new_args.size(), new_args.c_ptr()));
+    result = m_util.mk_bv_not(m_util.mk_bv_or(new_args.size(), new_args.data()));
     return BR_REWRITE3;
 }
 
@@ -2026,7 +2052,7 @@ br_status bv_rewriter::mk_bv_nand(unsigned num, expr * const * args, expr_ref &
     for (unsigned i = 0; i < num; i++) {
         new_args.push_back(m_util.mk_bv_not(args[i]));
     }
-    result = m_util.mk_bv_or(new_args.size(), new_args.c_ptr());
+    result = m_util.mk_bv_or(new_args.size(), new_args.data());
     return BR_REWRITE2;
 }
 
@@ -2293,16 +2319,15 @@ br_status bv_rewriter::mk_bit2bool(expr * lhs, expr * rhs, expr_ref & result) {
         return BR_DONE;
     }
 
-    if (m().is_ite(lhs)) {
-        result = m().mk_ite(to_app(lhs)->get_arg(0),
-                            m().mk_eq(to_app(lhs)->get_arg(1), rhs),
-                            m().mk_eq(to_app(lhs)->get_arg(2), rhs));
+    expr* a = nullptr, *b = nullptr, *c = nullptr;
+    if (m().is_ite(lhs, a, b, c)) {
+        result = m().mk_ite(a, m().mk_eq(b, rhs), m().mk_eq(c, rhs));
         return BR_REWRITE2;
     }
-
-    if (m_util.is_bv_not(lhs)) {
+    
+    if (m_util.is_bv_not(lhs, a)) {
         SASSERT(v.is_one() || v.is_zero());
-        result = m().mk_eq(to_app(lhs)->get_arg(0), mk_numeral(numeral(1) - v, 1));
+        result = m().mk_eq(a, mk_numeral(numeral(1) - v, 1));
         return BR_REWRITE1;
     }
 
@@ -2313,11 +2338,9 @@ br_status bv_rewriter::mk_bit2bool(expr * lhs, expr * rhs, expr_ref & result) {
 
     if (m_util.is_bv_or(lhs)) {
         ptr_buffer<expr> new_args;
-        unsigned num = to_app(lhs)->get_num_args();
-        for (unsigned i = 0; i < num; i++) {
-            new_args.push_back(m().mk_eq(to_app(lhs)->get_arg(i), bit1));
-        }
-        result = m().mk_or(new_args.size(), new_args.c_ptr());
+        for (expr* arg : *to_app(lhs))
+            new_args.push_back(m().mk_eq(arg, bit1));
+        result = m().mk_or(new_args);
         if (is_one) {
             return BR_REWRITE2;
         }
@@ -2330,12 +2353,10 @@ br_status bv_rewriter::mk_bit2bool(expr * lhs, expr * rhs, expr_ref & result) {
 
     if (m_util.is_bv_xor(lhs)) {
         ptr_buffer<expr> new_args;
-        unsigned num = to_app(lhs)->get_num_args();
-        for (unsigned i = 0; i < num; i++) {
-            new_args.push_back(m().mk_eq(to_app(lhs)->get_arg(i), bit1));
-        }
+        for (expr* arg : *to_app(lhs))
+            new_args.push_back(m().mk_eq(arg, bit1));
         // TODO: bool xor is not flat_assoc... must fix that.
-        result = m().mk_xor(new_args.size(), new_args.c_ptr());
+        result = m().mk_xor(new_args);
         if (is_one) {
             return BR_REWRITE2;
         }
@@ -2372,7 +2393,7 @@ br_status bv_rewriter::mk_blast_eq_value(expr * lhs, expr * rhs, expr_ref & resu
                                      mk_numeral(bit0 ? 0 : 1, 1)));
         div(v, two, v);
     }
-    result = m().mk_and(new_args.size(), new_args.c_ptr());
+    result = m().mk_and(new_args);
     return BR_REWRITE3;
 }
 
@@ -2438,7 +2459,7 @@ br_status bv_rewriter::mk_eq_concat(expr * lhs, expr * rhs, expr_ref & result) {
     }
     SASSERT(i1 == 0 && i2 == 0);
     SASSERT(new_eqs.size() >= 1);
-    result = m().mk_and(new_eqs.size(), new_eqs.c_ptr());
+    result = m().mk_and(new_eqs);
     return BR_REWRITE3;
 }
 
@@ -2525,7 +2546,7 @@ br_status bv_rewriter::mk_mul_eq(expr * lhs, expr * rhs, expr_ref & result) {
     unsigned sz;
     if (m_util.is_bv_mul(lhs, c, x) &&
         m_util.is_numeral(c, c_val, sz) &&
-        m_util.mult_inverse(c_val, sz, c_inv_val)) {
+        c_val.mult_inverse(sz, c_inv_val)) {
 
         SASSERT(m_util.norm(c_val * c_inv_val, sz).is_one());
 
@@ -2567,7 +2588,7 @@ br_status bv_rewriter::mk_mul_eq(expr * lhs, expr * rhs, expr_ref & result) {
         for (; !found && i < num_args; ++i) {
             expr* arg = to_app(rhs)->get_arg(i);
             if (m_util.is_bv_mul(arg, c2, x2) && m_util.is_numeral(c2, c2_val, sz) &&
-                m_util.mult_inverse(c2_val, sz, c2_inv_val)) {
+                c2_val.mult_inverse(sz, c2_inv_val)) {
                 found = true;
             }
         }
