@@ -42,6 +42,7 @@ Notes:
 #include "sat/sat_params.hpp"
 #include "sat/smt/euf_solver.h"
 #include "sat/tactic/goal2sat.h"
+#include "sat/tactic/sat2goal.h"
 #include "sat/tactic/sat_tactic.h"
 #include "sat/sat_simplifier_params.hpp"
 
@@ -660,30 +661,30 @@ public:
 
     void user_propagate_init(
         void*                ctx, 
-        solver::push_eh_t&   push_eh,
-        solver::pop_eh_t&    pop_eh,
-        solver::fresh_eh_t&  fresh_eh) override {
+        user_propagator::push_eh_t&   push_eh,
+        user_propagator::pop_eh_t&    pop_eh,
+        user_propagator::fresh_eh_t&  fresh_eh) override {
         ensure_euf()->user_propagate_init(ctx, push_eh, pop_eh, fresh_eh);
     }
         
-    void user_propagate_register_fixed(solver::fixed_eh_t& fixed_eh) override {
+    void user_propagate_register_fixed(user_propagator::fixed_eh_t& fixed_eh) override {
         ensure_euf()->user_propagate_register_fixed(fixed_eh);
     }
     
-    void user_propagate_register_final(solver::final_eh_t& final_eh) override {
+    void user_propagate_register_final(user_propagator::final_eh_t& final_eh) override {
         ensure_euf()->user_propagate_register_final(final_eh);
     }
     
-    void user_propagate_register_eq(solver::eq_eh_t& eq_eh) override {
+    void user_propagate_register_eq(user_propagator::eq_eh_t& eq_eh) override {
         ensure_euf()->user_propagate_register_eq(eq_eh);
     }
     
-    void user_propagate_register_diseq(solver::eq_eh_t& diseq_eh) override {
+    void user_propagate_register_diseq(user_propagator::eq_eh_t& diseq_eh) override {
         ensure_euf()->user_propagate_register_diseq(diseq_eh);
     }
     
-    unsigned user_propagate_register(expr* e) override { 
-        return ensure_euf()->user_propagate_register(e);
+    unsigned user_propagate_register_expr(expr* e) override { 
+        return ensure_euf()->user_propagate_register_expr(e);
     }
 
 private:
@@ -958,11 +959,11 @@ private:
         extract_asm2dep(asm2dep);
         sat::literal_vector const& core = m_solver.get_core();
         TRACE("sat",
-              for (auto kv : m_dep2asm) {
+              for (auto const& kv : m_dep2asm) {
                   tout << mk_pp(kv.m_key, m) << " |-> " << sat::literal(kv.m_value) << "\n";
               }
               tout << "asm2fml: ";
-              for (auto kv : asm2fml) {
+              for (auto const& kv : asm2fml) {
                   tout << mk_pp(kv.m_key, m) << " |-> " << mk_pp(kv.m_value, m) << "\n";
               }
               tout << "core: "; for (auto c : core) tout << c << " ";  tout << "\n";
@@ -1050,6 +1051,8 @@ private:
         eval.set_model_completion(true);
         bool all_true = true;
         for (expr * f : m_fmls) {
+            if (has_quantifiers(f))
+                continue;
             expr_ref tmp(m);
             eval(f, tmp);
             if (m.limit().is_canceled())
