@@ -289,34 +289,31 @@ namespace smt {
         */
         void user_propagate_init(
             void* ctx, 
-            solver::push_eh_t&      push_eh,
-            solver::pop_eh_t&       pop_eh,
-            solver::fresh_eh_t&     fresh_eh);
+            user_propagator::push_eh_t&      push_eh,
+            user_propagator::pop_eh_t&       pop_eh,
+            user_propagator::fresh_eh_t&     fresh_eh);
 
-        void user_propagate_register_fixed(solver::fixed_eh_t& fixed_eh);
+        void user_propagate_register_fixed(user_propagator::fixed_eh_t& fixed_eh);
 
-        void user_propagate_register_final(solver::final_eh_t& final_eh);
+        void user_propagate_register_final(user_propagator::final_eh_t& final_eh);
         
-        void user_propagate_register_eq(solver::eq_eh_t& eq_eh);
+        void user_propagate_register_eq(user_propagator::eq_eh_t& eq_eh);
         
-        void user_propagate_register_diseq(solver::eq_eh_t& diseq_eh);
+        void user_propagate_register_diseq(user_propagator::eq_eh_t& diseq_eh);
 
-
-        /**
-           \brief register an expression to be tracked fro user propagation.
-        */
-        unsigned user_propagate_register(expr* e);
+        unsigned user_propagate_register_expr(expr* e);
         
+        void user_propagate_register_created(user_propagator::created_eh_t& r);
 
         /**
            \brief Return a reference to smt::context.
-           This is a temporary hack to support user theories.
-           TODO: remove this hack.
-           We need to revamp user theories too.
+           This breaks abstractions. 
+           
+           It is currently used by the opt-solver
+           to access optimization services from arithmetic solvers
+           and to ensure that the solver has registered PB theory solver.
 
-           This method breaks the abstraction barrier.
-
-           \warning We should not use this method
+           \warning This method should not be used in new code.
         */
         context & get_context();
     };
